from h2o_wave import ui

INPUT_MODELS = ['en_core_web_sm', 'en_core_web_md']
INPUT_MODEL = 'en_core_web_sm'
INPUT_TEXT = 'Matthew Honnibal and Ines Montani are the founders of Explosion'

META_CARD = ui.meta_card(
    box='',
    title='WaCy',
    layouts=[
        ui.layout(
            breakpoint='xs',
            zones=[
                ui.zone(name='header'),
                ui.zone(name='main', zones=[
                    ui.zone(name='input_row', direction='row', zones=[
                        ui.zone(name='input_model', size='30%'),
                        ui.zone(name='input_text', size='70%')
                    ]),
                    ui.zone(name='dependency_row', direction='row', zones=[
                        ui.zone(name='dependency_settings', size='30%'),
                        ui.zone(name='dependency_visualizer', size='70%')
                    ]),
                    ui.zone(name='entity_row', direction='row', zones=[
                        ui.zone(name='entity_settings', size='30%'),
                        ui.zone(name='entity_visualizer', size='70%')
                    ])
                ]),
                ui.zone(name='footer')
            ]
        )
    ]
)

HEADER_CARD = ui.header_card(
    box='header',
    title='WaCy',
    subtitle='Powering spaCy with Wave',
    icon='WavingHand'
)

FOOTER_CARD = ui.footer_card(
    box='footer',
    caption='(c) 2021 <b>TheWaverGuy</b> - <i>Data is bulletproof</i>'
)



from wacy.cards.input import InputModelCard, InputTextCard
from wacy.cards.dependency import DependencySettingsCard, DependencyVisualizerCard
from wacy.cards.entity import EntitySettingsCard, EntityVisualizerCard



from typing import List

import spacy
from spacy import displacy
from h2o_wave import Q, ui
from h2o_wave.ui import Command

from .base import BaseCard


class EntitySettingsCard(BaseCard):
    def __init__(
        self,
        name: str = 'entity_settings',
        box: str = 'entity_settings',
        choice_ents: list = None,
        select_ents: list = None,
        title: str = 'Entity Settings',
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.choice_ents = choice_ents
        self.select_ents = select_ents
        self.title = title
        self.commands = commands

    def to_displacy_options(self):
        return {
            'ents': self.select_ents
        }

    async def render(self, q: Q):
        entity_choices = [ui.choice(name=str(x), label=str(x)) for x in self.choice_ents]

        card = ui.form_card(
            box=self.box,
            items=[
                ui.picker(
                    name='select_ents',
                    label='Entities',
                    choices=entity_choices,
                    values=self.select_ents,
                    trigger=True
                )
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card


class EntityVisualizerCard(BaseCard):
    def __init__(
        self,
        name: str = 'entity_visualizer',
        box: str = 'entity_visualizer',
        doc: spacy.tokens.Doc = None,
        options: dict = None,
        title: str = 'Entity Visualizer',
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.doc = doc
        self.options = options
        self.title = title
        self.commands = commands

    async def render(self, q: Q):
        dependency_html = displacy.render(self.doc, style='ent', options=self.options)

        card = ui.form_card(
            box=self.box,
            items=[
                ui.text(content=dependency_html)
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card



from typing import List

import spacy
from spacy import displacy
from h2o_wave import Q, ui
from h2o_wave.ui import Command

from .base import BaseCard


class DependencySettingsCard(BaseCard):
    def __init__(
        self,
        name: str = 'dependency_settings',
        box: str = 'dependency_settings',
        fine_grained: bool = False,
        add_lemma: bool = False,
        collapse_punct: bool = True,
        collapse_phrases: bool = False,
        compact: bool = False,
        color: str = 'green',
        bg: str = 'white',
        font: str = 'Arial',
        offset_x: int = 50,
        arrow_stroke: int = 2,
        arrow_width: int = 9,
        arrow_spacing: int = 11,
        word_spacing: int = 45,
        word_distance: int = 175,
        title: str = 'Dependency Settings',
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.fine_grained = fine_grained
        self.add_lemma = add_lemma
        self.collapse_punct = collapse_punct
        self.collapse_phrases = collapse_phrases
        self.compact = compact
        self.color = color
        self.bg = bg
        self.font = font
        self.offset_x = offset_x
        self.arrow_stroke = arrow_stroke
        self.arrow_width = arrow_width
        self.arrow_spacing = arrow_spacing
        self.word_spacing = word_spacing
        self.word_distance = word_distance
        self.title = title
        self.commands = commands

    def to_displacy_options(self):
        return {
            'fine_grained': self.fine_grained,
            'add_lemma': self.add_lemma,
            'collapse_punct': self.collapse_punct,
            'collapse_phrases': self.collapse_phrases,
            'compact': self.compact,
            'color': self.color,
            'bg': self.bg,
            'font': self.font,
            'offset_x': self.offset_x,
            'arrow_stroke': self.arrow_stroke,
            'arrow_width': self.arrow_width,
            'arrow_spacing': self.arrow_spacing,
            'word_spacing': self.word_spacing,
            'word_distance': self.word_distance
        }

    async def render(self, q: Q):
        card = ui.form_card(
            box=self.box,
            items=[
                ui.toggle(name='fine_grained', label='Fine Grained', value=self.fine_grained, trigger=True),
                ui.toggle(name='add_lemma', label='Add Lemma', value=self.add_lemma, trigger=True),
                ui.toggle(name='collapse_punct', label='Merge Punctuation', value=self.collapse_punct, trigger=True),
                ui.toggle(name='collapse_phrases', label='Merge Phrases', value=self.collapse_phrases, trigger=True),
                ui.toggle(name='compact', label='Compact', value=self.compact, trigger=True),
                ui.inline(items=[
                    ui.textbox(name='color', label='Visualizer Color', value=self.color, trigger=True),
                    ui.textbox(name='bg', label='Background Color', value=self.bg, trigger=True)
                ]),
                ui.inline(items=[
                    ui.textbox(name='font', label='Font', value=self.font, trigger=True),
                    ui.textbox(name='offset_x', label='Offset-X', value=str(self.offset_x), trigger=True)
                ]),
                ui.inline(items=[
                    ui.textbox(
                        name='arrow_stroke',
                        label='Arrow Stroke',
                        value=str(self.arrow_stroke),
                        suffix='px',
                        trigger=True
                    ),
                    ui.textbox(
                        name='arrow_width',
                        label='Arrow Width',
                        value=str(self.arrow_width),
                        suffix='px',
                        trigger=True
                    ),
                    ui.textbox(
                        name='arrow_spacing',
                        label='Arrow Spacing',
                        value=str(self.arrow_spacing),
                        suffix='px',
                        trigger=True
                    )
                ]),
                ui.inline(items=[
                    ui.textbox(
                        name='word_spacing',
                        label='Word Spacing',
                        value=str(self.word_spacing),
                        suffix='px',
                        trigger=True
                    ),
                    ui.textbox(
                        name='word_distance',
                        label='Word Distance',
                        value=str(self.word_distance),
                        suffix='px',
                        trigger=True
                    )
                ])
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card


class DependencyVisualizerCard(BaseCard):
    def __init__(
        self,
        name: str = 'dependency_visualizer',
        box: str = 'dependency_visualizer',
        doc: spacy.tokens.Doc = None,
        options: dict = None,
        title: str = 'Dependency Visualizer',
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.doc = doc
        self.options = options
        self.title = title
        self.commands = commands

    async def render(self, q: Q):
        dependency_html = displacy.render(self.doc, style='dep', options=self.options)

        card = ui.form_card(
            box=self.box,
            items=[
                ui.text(content=dependency_html)
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card



from typing import List

from h2o_wave import Q, ui
from h2o_wave.ui import Command

from .base import BaseCard
from ..utils import defaults


class InputModelCard(BaseCard):
    def __init__(
        self,
        name: str = 'input_model',
        box: str = 'input_model',
        input_models: List[str] = None,
        input_model: str = defaults.INPUT_MODEL,
        title: str = None,
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.input_models = input_models if input_models is not None else defaults.INPUT_MODELS
        self.input_model = input_model
        self.title = title
        self.commands = commands

    async def render(self, q: Q):
        model_choices = [ui.choice(name=str(x), label=str(x)) for x in self.input_models]

        card = ui.form_card(
            box=self.box,
            items=[
                ui.dropdown(
                    name='input_model',
                    label='Select model',
                    value=self.input_model,
                    choices=model_choices,
                    trigger=True
                )
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card


class InputTextCard(BaseCard):
    def __init__(
        self,
        name: str = 'input_text',
        box: str = 'input_text',
        input_text: str = defaults.INPUT_TEXT,
        title: str = None,
        commands: List[Command] = None
    ):
        super().__init__(name, box)

        self.input_text = input_text
        self.title = title
        self.commands = commands

    async def render(self, q: Q):
        card = ui.form_card(
            box=self.box,
            items=[
                ui.textbox(name='input_text', label='Enter text to analyze', value=self.input_text, multiline=True),
                ui.button(name='analyze_text', label='Analyze', primary=True)
            ],
            title=self.title,
            commands=self.commands
        )

        q.page[self.name] = card



from h2o_wave import Q, ui


class BaseCard(object):
    def __init__(self, name, box):
        self.name = name
        self.box = box

    async def render(self, q: Q):
        q.page[self.name] = ui.form_card(box=self.box, items=[])



import setuptools

with open('VERSION.txt', 'r') as f:
    version = f.read()

with open('README.md', 'r') as f:
    long_description = f.read()

setuptools.setup(
    name='wacy',
    version=version,
    author='TheWaverGuy',
    author_email='thewaverguy@gmail.com',
    description='Powering spaCy with Wave',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/thewaverguy/wacy',
    project_urls={
        'Source': 'https://github.com/thewaverguy/wacy',
        'Tracker': 'https://github.com/thewaverguy/wacy'
    },
    python_requires='>=3.7',
    install_requires=[
        'h2o_wave>=0.13.0',
        'spacy>=3.0.0'
    ],
    packages=setuptools.find_packages(),
    license='Apache License, Version 2.0',
    platforms='any',
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'License :: OSI Approved :: Apache Software License',
        'Operating System :: OS Independent',
    ],
    keywords=[
        'app',
        'h2o_wave',
        'nlp',
        'spacy'
    ]
)
